/*
 * Private device independent routines for display of Life objects.
 * These can handle dumb terminals, terminals using curses routines,
 * and X displays.  Nicer interface routines are in dev.h.
 *
 * Copyright (c) 2014 by David I. Bell
 * Permission is granted to use, distribute, or modify this source,
 * provided that this copyright notice remains intact.
 */

#ifndef	DEVPRIV_H
#define	DEVPRIV_H

#include "hashlife.h"


#define	STDIN	0
#define	STDOUT	1
#define	STDERR	2


typedef	int		COUNT;
typedef	unsigned char	UCHAR;
typedef struct DEV	DEV;


/*
 * Structure to interface to various input/output devices.
 * This supports normal terminals and graphics output.
 */
struct DEV
{
	BOOL	(*open)(DEV *);
	void	(*close)(DEV *);
	void	(*update)(DEV *);
	void	(*refresh)(DEV *);

	int	(*peekInput)(DEV *);
	int	(*readChar)(DEV *, BOOL isWait);
	BOOL	(*isInputReady)(DEV *);
	BOOL	(*isRedrawNeeded)(DEV *);

	BOOL	(*setMag)(DEV *, int mag);
	void	(*viewBegin)(DEV *);
	void	(*viewCells)(DEV *, int count, int state);
	void	(*viewNextLine)(DEV *);
	void	(*viewEnd)(DEV *);

	void	(*showStatus)(DEV *, const char * msg);
	void	(*addStatus)(DEV *, const char * msg);

	void	(*beginText)(DEV *);
	void	(*addText)(DEV *, const char * msg);
	void	(*endText)(DEV *);

	COUNT	rows;
	COUNT	cols;
	COUNT	textRows;
	COUNT	textCols;
	int	defaultMag;
};


/*
 * The device being used.
 */
extern	DEV * dev;


/*
 * Devices which we may use.
 */
#ifdef	TTY
extern	DEV ttyDev;
#endif

#ifdef	CURSES
extern	DEV cursesDev;
#endif

#ifdef	X11
extern	DEV x11Dev;
#endif

#endif

